#include "stdafx.h"
#include "Global.h"

// -- definitions --

// GUID created for this.
const TCHAR *szwaGenScriptingWindowClass = "gen_scripting_server_917F7CC4-2865-4df2-9194-3E9FD9466FBE";
const TCHAR *szwagsReplyWindowClass      = "gen_scripting_reply_window";

// -- utility functions --

// delete[] the result.
// Returns NULL on failure.
TCHAR *LeoDavidsonUtils::StringAllocAndFormat(const TCHAR *format, ...)
{
	va_list args;
	va_start(args, format);
	TCHAR *result = VStringAllocAndFormat(format, args);
	va_end(args);

	return(result);
}

// delete[] the result.
// Returns NULL on failure.
TCHAR *LeoDavidsonUtils::VStringAllocAndFormat(const TCHAR *format, va_list args)
{
	size_t bufferSize = BUFFERSIZE;

	TCHAR *szResult = new TCHAR[bufferSize];

	while(true)
	{
		int formatRes = _vsntprintf(szResult, bufferSize, format, args);

		// formatRes negative on error; (bufferSize == formatRes) should be redundant for _vsntprintf.

		if ((0 > formatRes) || (bufferSize == formatRes))
		{
			delete [] szResult;
			bufferSize *= 2;
			szResult = new TCHAR[bufferSize];
		}
		else
		{
			break;
		}
	}

	return(szResult);
}

// Returns NULL on failure (use GetLastError())
// delete[] the result when finished with it.
WCHAR *LeoDavidsonUtils::LeetMBtoWC(const char *mbString)
{
	WCHAR *result = NULL;

	int wcBufSize = MultiByteToWideChar(CP_ACP, 0, mbString, -1, NULL, 0);

	result = new WCHAR[wcBufSize];

	// Hopefully this includes the '\0' in the copy and the count of characters,
	// else this will not work with empty strings.
	if (0 == MultiByteToWideChar(CP_ACP, 0, mbString, -1, result, wcBufSize))
	{
		delete [] result;
		result = NULL;
	}

	return(result);
}

// Returns NULL on failure (use GetLastError())
// delete[] the result when finished with it.
char *LeoDavidsonUtils::LeetWCtoMB(const WCHAR *wcString)
{
	char *result = NULL;

	int mbBufSize = WideCharToMultiByte(CP_ACP, 0, wcString, -1, NULL, 0, NULL, NULL);

	result = new char[mbBufSize];

	// Hopefully this includes the '\0' in the copy and the count of characters,
	// else this will not work with empty strings.
	if (0 == WideCharToMultiByte(CP_ACP, 0, wcString, -1, result, mbBufSize, NULL, NULL))
	{
		delete [] result;
		result = NULL;
	}

	return(result);
}

std::wstring LeoDavidsonUtils::LeetMBtoWCString(const char *mbString)
{
	WCHAR *wcString = LeetMBtoWC(mbString);

	std::wstring strResult = wcString;

	delete [] wcString;

	return(strResult);
}

std::string LeoDavidsonUtils::LeetWCtoMBString(const WCHAR *wcString)
{
	char *mbString = LeetWCtoMB(wcString);

	std::string strResult = mbString;

	delete [] mbString;

	return(strResult);
}
